<?php
// $Id: voting_actions_ui.inc,v 1.1.2.6 2007/04/20 19:46:32 eaton Exp $

// Contains UI management functions for the voting_actions module.

function voting_actions_admin_page() {
  voting_actions_get_sets(TRUE);
  $db_sets = _voting_actions_load_sets_from_db();
  foreach ($db_sets as $name => $set) {
    $vasid = $set['vasid'];
    $links = array();
    $links['voting_actions_edit'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/gtk-edit.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/edit/$name",
       'attributes' => array('title' => t('Edit this set')),
    );
    $links['voting_actions_export'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/document-save.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/export/$name",
       'attributes' => array('title' => t('Export this set')),
    );
    $links['voting_actions_delete'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/edit-delete.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/delete/$name",
       'attributes' => array('title' => t('Delete this set')),
    );

    $items[] = array($name, theme('links', $links));
  }

  if (empty($items)) {
    $items[] = array('data' => array(array('data' => '<p class="description">' .t("No action sets have currently been defined. !click to create one.", array('!click' => l(t('Click here'), 'admin/settings/voting_actions/add'))) .'</p>', 'colspan' => 2)));
  }
  $output = theme('table', array(t('Name'), t('Actions')), $items, array("cellpadding" => "3"), t('User-Defined Action Sets'));

  $items = array();
  $module_sets = module_invoke_all('voting_actions_sets');

  foreach ($module_sets as $name => $set) {
    $set += _voting_actions_populate_set_defaults();    
    $set['status'] = voting_actions_get_status($name);

    $links = array();
    if (array_key_exists($name, $db_sets) && $set['status'] != VOTING_ACTIONS_STATE_OVERRIDDEN) {
      voting_actions_set_status($name, VOTING_ACTIONS_STATE_OVERRIDDEN);
      $set['status'] = voting_actions_get_status($name);
    }
    else {
      $links['voting_actions_add'] = array(
       'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/gtk-edit.png'),
       'html' => TRUE,
       'href' => "admin/settings/voting_actions/edit/$name",
       'attributes' => array('title' => t('Edit this set')),
      );
      $links['voting_actions_export'] = array(
         'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/document-save.png'),
         'html' => TRUE,
         'href' => "admin/settings/voting_actions/export/$name/mod",
         'attributes' => array('title' => t('Export this set')),
      );
    }

    switch($set['status']) {
      case VOTING_ACTIONS_STATE_ENABLED:
        $toggle['voting_actions_disable'] = array(
          'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/media-playback-start.png') . t('Enabled'),
          'html' => TRUE,
          'href' => "admin/settings/voting_actions/disable/$name",
          'attributes' => array('title' => t('Disable this set')),
        );
        break;
      case VOTING_ACTIONS_STATE_DISABLED:
        $toggle['voting_actions_enable'] = array(
          'title' => theme('image', drupal_get_path('module', 'voting_actions') . '/icons/process-stop.png') . t('Disabled'),
          'html' => TRUE,
          'href' => "admin/settings/voting_actions/enable/$name",
          'attributes' => array('title' => t('Enable this set')),
        );
        break;
      default:
        $toggle['voting_actions_enable'] = array(
          'title' => t('Overridden'),
          'attributes' => array('title' => t('Overridden')),
        );
        break;
    }

    $items[] = array($name, theme('links', $toggle), theme('links', $links));
  }

  if (empty($items)) {
    $items[] = array('data' => array(array('data' => '<p class="description">'. t('No default action sets have been exposed.') .'</p>', 'colspan' => '4')));
  }
  $output .= theme('table', array(t('Name'), t('Status'), t('Actions')), $items, array("cellpadding" => "3"), t('Default Action Sets'));

  return $output;
}

function voting_actions_admin_import_page($set_name = NULL) {
  $form['set'] = array(
    '#type' => 'textarea',
    '#title' => t('Import action set'),
    '#rows' => 20
  );
  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t("Submit"),
  );

  return $form;
}
function voting_actions_admin_import_page_validate($form_id, &$form) {
  ob_start();
  eval($form['set']);
  ob_end_clean();

  if (is_array($sets)) {
    foreach($sets as $name => $set) {
      if ($name) {
        $errors = _voting_actions_validate_set($set);
        foreach($errors as $error) {
          form_set_error('', $error);
        }
      }
    }
  }
}

function voting_actions_admin_import_page_submit($formid, $form) {
  ob_start();
  eval($form['set']);
  ob_end_clean();

  if (is_array($sets)) {
    foreach($sets as $name => $set) {
      if ($name) {
        $errors = _voting_actions_validate_set($set);
        if (count($errors) == 0) {
          _voting_actions_insert_set($set);
          return 'admin/settings/voting_actions';
        }
        else {
          foreach ($errors as $error) {
            drupal_set_message($error);
            return FALSE;
          }
        }
      }
      else {
        drupal_set_message(t('Unable to get an action set out of that.'));
        return FALSE;
      }
    }
  }
}


function voting_actions_admin_export_page($set_name) {
  $code = _voting_actions_create_set_code($set_name);
  $lines = substr_count($code, "\n");
  $form['code'] = array(
    '#type' => 'textarea',
    '#title' => $set['name'],
    '#default_value' => $code,
    '#rows' => max($lines, 20)
  );
  return $form;
}

function _voting_actions_create_set_code($set_name) {
  $data = cache_get('voting_actions_cache');
  $action_sets = unserialize($data->data);
  $set = $action_sets[$set_name];

  unset($set['vasid']);
  foreach($set['conditions'] as $key => $condition) {
    unset($set['conditions'][$key]['vacid']);
    unset($set['conditions'][$key]['vasid']);
  }

  if (!empty($set)) {
    $code .= '$set = ';
    ob_start();
    var_export($set);
    $code .= ob_get_clean();
    ob_end_clean();
    
    $code .= "\n\$sets['$set_name'] = \$set;";
    $code .= "\nreturn \$sets;";
  }
  
  return $code;
}


function voting_actions_admin_delete_page($set_name) {
  $form['set'] = array('#type' => 'value', '#value' => $set_name);
  return confirm_form(
    $form, 
    t('Are you sure you want to delete %title?', array('%title' => $set_name)), 
    $_GET['destination'] ? $_GET['destination'] : 'admin/settings/voting_actions', 
    t('This action cannot be undone.'),
    t('Delete'), t('Cancel')  
  );
}
function voting_actions_admin_delete_page_submit($formid, $form) {
  $name = $form['set'];
  $sets = _voting_actions_load_sets_from_db();
  if ($set = $sets[$name]) {
    _voting_actions_delete_set($set);
  }
  return 'admin/settings/voting_actions';
}



function voting_actions_admin_enable_page($set_name) {
  voting_actions_set_status($set_name, VOTING_ACTIONS_STATE_ENABLED);
  drupal_goto('admin/settings/voting_actions');
}

function voting_actions_admin_disable_page($set_name) {
  voting_actions_set_status($set_name, VOTING_ACTIONS_STATE_DISABLED);
  drupal_goto('admin/settings/voting_actions');
}

function voting_actions_admin_edit_page($name = '') {
  $sets = voting_actions_get_sets();
  $set = empty($sets[$name]) ? _voting_actions_populate_set_defaults() : $sets[$name];
  return drupal_get_form('voting_actions_set_form', $set);
}


function voting_actions_set_form($set, $form_values = NULL) {
  if (isset($form_values)) {
    $set = _voting_actions_set_from_values($set, $form_values);
    
    if ($form_values['op'] == t('Add condition')) {
      $set['conditions'][] = array('name' => $form_values['conditions']['add']['condition_type']);
    }

    if ($form_values['op'] == t('Add action')) {
      $set['actions'][] = $form_values['actions']['add']['action_type'];
    }

    if (!empty($form_values['del_cond_op'])) {
      unset($set['conditions'][($form_values['del_cond_op'] - 1)]);
    }
  
    if (!empty($form_values['del_action_op'])) {
      unset($set['actions'][($form_values['del_action_op'] - 1)]);
    }
  }

  $form = _voting_actions_set_form($set);

  $form['conditions'] =  _voting_actions_set_conditions_form($set);

  $form['actions'] =  _voting_actions_set_actions_form($set);

  $form['submit'] = array(
    '#type' => 'submit',
    '#value' => t('Submit'),
  );

  if (!empty($set['vasid'])) {
    $form['delete'] = array(
      '#type' => 'submit',
      '#value' => t('Delete'),
    );
  }

  return $form;
}

function voting_actions_set_form_validate($form_id, $form_values) {
  if ($form_values['op'] == t('Submit')) {
  
    if (empty($form_values['name'])) {
      form_set_error('name', t('Action sets must be named.'));
    }
    elseif ($form_values['name'] != strtolower(str_replace(array(' ', '&', '%', '#', '$', '@', '.', ','), '_', $form_values['name']))) {
      form_set_error('name', t('The set name must consist of lowercase letters with no spaces or special symbols -- use underscores or dashes instead of spaces.'));
    }
    elseif(empty($form_values['vasid'])) {
      if (db_result(db_query("SELECT 1 FROM {voting_actions_set} WHERE name = '%s'", $form_values['name']))) {
        form_set_error('name', t('A voting action set with the name @setname already exists. Choose a new name.', array('@setname' => $form_values['name'])));
      }
    }
    
    $set = _voting_actions_set_from_values(array(), $form_values);
    $errors = _voting_actions_validate_set($set);
    foreach ($errors as $error) {
      form_set_error(NULL, $error);
    }
  }
}

function voting_actions_set_form_submit($form_id, $form_values) {
  if ($form_values['op'] == t('Submit')) {
    $set = _voting_actions_set_from_values(array(), $form_values);
    if (empty($set['vasid'])) {
     _voting_actions_insert_set($set);
    }
    else {
      _voting_actions_update_set($set);
    }
    
    drupal_set_message(t('Your voting action set was saved.'));
    return 'admin/settings/voting_actions/';
  }
  elseif ($form_values['op'] == t('Delete')) {
    return 'admin/settings/voting_actions/delete/'. $form_values['name'];
  }
  else {
    return FALSE;
  }
}

function _voting_actions_set_from_values($set = array(), $form_values = array()) {
    // Build an in-process set before going any further.
    $set['name'] = $form_values['name'];
    $set['description'] = $form_values['description'];
    $set['parent'] = $form_values['parent'];
    $set['source'] = $form_values['source'];
    $set['required'] = $form_values['required'];
    $set['weight'] = $form_values['weight'];
    $set['condition_mask'] = $form_values['conditions']['condition_mask'];
    if (!empty($form_values['vasid'])) {
      $set['vasid'] = $form_values['vasid'];
    }

    $set += array(
      'conditions' => array(),
      'actions' => array(),
      'subsets' => array(),
    );

    if (!empty($form_values['conditions']['existing'])) {
      $set['conditions'] = $form_values['conditions']['existing'];
    }
    if (!empty($form_values['actions']['existing'])) {
      foreach($form_values['actions']['existing'] as $key => $element) {
        $set['actions'][$key] = $form_values['actions']['existing'][$key]['name'];
      }
    }
    
    return $set;
}

function _voting_actions_set_form($set) {
  $form = array();

  $form['#multistep'] = TRUE;
  $form['#tree'] = TRUE;

  $form['vasid'] = array(
    '#type' => 'hidden',
    '#value' => $set['vasid'],
   );

  $form['name'] = array(
    '#type' => 'textfield',
    '#title' => t('Set name'),
    '#description' => t('A unique name for this set. Stick to lowercase letters, and use underscores instead of spaces.'),
    '#default_value' => $set['name'],
  );
  
  $form['description'] = array(
    '#type' => 'textfield',
    '#title' => t('Description'),
    '#description' => t("A description of this action set's operations. Useful for documenting the intended purpose of the set."),
    '#default_value' => $set['description'],
  );
  
  if (!empty($set['parent'])) {
    $form['parent'] = array(
      '#type' => 'hidden',
      '#value' => $set['parent'],
    );
    $form['required'] = array(
      '#type' => 'checkbox',
      '#title' => t('This set is required.'),
      '#return_value' => 1,
      '#default_value' => $set['required'],
      '#description' => t("If this box is checked, this set's conditions must be passed before the parent set's actions can execute."),
    );
  }
  else {
    $form['required'] = array(
      '#type' => 'hidden',
      '#value' => TRUE,
    );
  }

  $form['weight'] = array(
    '#type' => 'hidden',
    '#value' => 0,
  );


  return $form;
}
function _voting_actions_set_conditions_form($set) {
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Conditions'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
  );

  $form['condition_mask'] = array(
    '#type' => 'radios',
    '#options' => array('AND' => t('All conditions must be true'), 'OR' => t('At least one condition must be true')),
    '#default_value' => isset($set['condition_mask']) ? $set['condition_mask'] : 'AND',
  );

  $options = array();
  foreach (voting_actions_get_conditions() as $key => $data) {
    $options[$key] = $data['name'];
  }

  $c = 0;
  $form['existing']['#weight'] = 0;
  foreach($set['conditions'] as $condition) {
    $form['existing'][$c++] = voting_actions_condition_form($condition, $c, count($set['conditions']));
  }

  $form['add'] = array(
    '#type' => 'item',
    '#weight' => 100,
    '#title' => t('Add a condition'),
    '#theme' => 'voting_actions_inline_form',
  );
  $form['add']['condition_type'] = array(
    '#type' => 'select',
    '#options' => $options,
  );
  $form['add']['add_condition'] = array(
    '#type' => 'submit',
    '#value' => t('Add condition'),
  );
  return $form;
}

function _voting_actions_set_actions_form($set) {
  $form = array(
    '#type' => 'fieldset',
    '#title' => t('Actions'),
    '#collapsible' => TRUE,
    '#collapsed' => FALSE,
    '#description' => t('If the conditions above pass, this list of actions will execute.'),
  );

  $a = 0;
  $form['existing']['#weight'] = 0;
  foreach($set['actions'] as $action) {
    $form['existing'][$a++] = voting_actions_action_form($action, $a, count($set['actions']));
  }

  $form['add'] = array(
    '#type' => 'item',
    '#weight' => 100,
    '#title' => t('Add an action'),
    '#theme' => 'voting_actions_inline_form',
  );
  $form['add']['action_type'] = array(
    '#type' => 'select',
    '#options' => _voting_actions_list(),
  );
  $form['add']['add_action'] = array(
    '#type' => 'submit',
    '#value' => t('Add action'),
  );
  return $form;
}

function voting_actions_condition_form($condition, $c, $count) {
  $form = array();
  $conditions = voting_actions_get_conditions();
  if ($condition_info = $conditions[$condition['name']]) {
    $form = array(
      '#type' => 'item',
      '#title' => $condition_info['name'],
      '#description' => $condition_info['description'],
      '#attributes' => array('class' => 'container-inline'),
      '#theme' => 'voting_actions_inline_form',
    );

    if (!empty($condition['vacid'])) {
      $form['vacid'] = array(
        '#type' => 'hidden',
        '#value' => $condition['vacid'],
      );
    }

    $form['name'] = array(
      '#type' => 'hidden',
      '#value' => $condition['name'],
    );

    $form['weight'] = array(
      '#type' => 'hidden',
      '#value' => $condition['weight'],
    );

    if (($function = $condition_info['edit_handler']) && function_exists($function)) {
      $form['data'] = $function($condition['data']);
    }


/* Baby steps. Let's handle reordering later.
    if ($c > 1) {
      $form['up'] = array(
        '#type' => 'imagebutton',
        '#name' => 'up_cond_op',
        '#default_value' => $c,
        '#title' => t('Move up'),
        '#image' => drupal_get_path('module', 'voting_actions') .'/icons/go-up.png',
      );
    }

    if ($c < $count) {
      $form['down'] = array(
        '#type' => 'imagebutton',
        '#name' => 'down_cond_op',
        '#default_value' => $c,
        '#title' => t('Move down'),
        '#image' => drupal_get_path('module', 'voting_actions') .'/icons/go-down.png',
      );
    }
*/    
    $form['del'] = array(
      '#type' => 'imagebutton',
      '#name' => 'del_cond_op',
      '#default_value' => $c,
      '#title' => t('Delete this criteria'),
      '#image' => drupal_get_path('module', 'voting_actions') .'/icons/edit-delete.png',
    );
  }
  return $form;
}

function voting_actions_action_form($action, $a, $count) {
  $form = array(
    '#theme' => 'voting_actions_inline_form',
  );
  
  $actions = actions_get_all_actions();

  $form['name'] = array(
    '#type' => 'hidden',
    '#value' => $action,
  );
  $form['description'] = array(
    '#value' => $actions[$action]['description'],
  );

  $form['del'] = array(
    '#type' => 'imagebutton',
    '#name' => 'del_action_op',
    '#default_value' => $a,
    '#title' => t('Delete this action'),
    '#image' => drupal_get_path('module', 'voting_actions') .'/icons/edit-delete.png',
  );
    
  return $form;
}

function _voting_actions_list() {
  $options = array();
  foreach (actions_get_all_actions() as $aid => $action) {
    $options[$action['type']][$aid] = $action['description'];
  }
  return $options;
}

function theme_voting_actions_inline_form($form) {
  return '<div class="container-inline">'. drupal_render($form) .'</div>';
}

/**
 * Custom form element to do nice image buttons.
 */
function voting_actions_elements() {
  $type['imagebutton'] = array('#input' => TRUE, '#name' => 'op', '#button_type' => 'submit',);
  return $type;
}

function theme_imagebutton($element) {
  return '<input type="image" class="form-'. $element['#button_type'] .'" name="'. $element['#name'] .'" id="'. $element['#id'] .'" value="'. check_plain($element['#default_value']) .'" '. drupal_attributes($element['#attributes']) . ' src="/' . $element['#image'] . '" alt="' . $element['#title'] . '" title="' . $element['#title'] . "\" />\n";
}

function imagebutton_value() {
  // Do-nothing function ensures default value doesn't migrate to #value
}